@@ -10,7 +10,7 @@ use serde::{Serialize, Deserialize};
 type ServiceError = Box<dyn Error + Send + Sync + 'static>;
 
 pub const BASE_PATH: &'static str = "";
-pub const API_VERSION: &'static str = "0.2.0";
+pub const API_VERSION: &'static str = "0.3.0";
 
 #[derive(Debug, PartialEq, Serialize, Deserialize)]
 #[must_use]
@@ -219,6 +219,75 @@ pub enum UpdateParishResponse {
     ValidationException
 }
 
+#[derive(Debug, PartialEq, Serialize, Deserialize)]
+#[must_use]
+pub enum AddProvinceResponse {
+    /// successful operation
+    SuccessfulOperation
+    (models::Province)
+    ,
+    /// Invalid input
+    InvalidInput
+}
+
+#[derive(Debug, PartialEq, Serialize, Deserialize)]
+#[must_use]
+pub enum DeleteProvinceResponse {
+    /// successful operation
+    SuccessfulOperation
+    ,
+    /// Invalid ID supplied
+    InvalidIDSupplied
+    ,
+    /// Province not found
+    ProvinceNotFound
+}
+
+#[derive(Debug, PartialEq, Serialize, Deserialize)]
+#[must_use]
+pub enum GetProvinceResponse {
+    /// successful operation
+    SuccessfulOperation
+    (models::ProvinceCollection)
+    ,
+    /// Invalid ID supplied
+    InvalidIDSupplied
+    ,
+    /// Province not found
+    ProvinceNotFound
+}
+
+#[derive(Debug, PartialEq, Serialize, Deserialize)]
+#[must_use]
+pub enum GetProvinceByIdResponse {
+    /// successful operation
+    SuccessfulOperation
+    (models::Province)
+    ,
+    /// Invalid ID supplied
+    InvalidIDSupplied
+    ,
+    /// Province not found
+    ProvinceNotFound
+}
+
+#[derive(Debug, PartialEq, Serialize, Deserialize)]
+#[must_use]
+pub enum UpdateProvinceResponse {
+    /// successful operation
+    SuccessfulOperation
+    (models::Province)
+    ,
+    /// Invalid ID supplied
+    InvalidIDSupplied
+    ,
+    /// Province not found
+    ProvinceNotFound
+    ,
+    /// Validation exception
+    ValidationException
+}
+
 /// API
 #[async_trait]
 pub trait Api<C: Send + Sync> {
@@ -325,6 +394,37 @@ pub trait Api<C: Send + Sync> {
         parish: Option<models::Parish>,
         context: &C) -> Result<UpdateParishResponse, ApiError>;
 
+    /// Add new province
+    async fn add_province(
+        &self,
+        province: Option<models::Province>,
+        context: &C) -> Result<AddProvinceResponse, ApiError>;
+
+    /// Delete province by ID
+    async fn delete_province(
+        &self,
+        id: uuid::Uuid,
+        context: &C) -> Result<DeleteProvinceResponse, ApiError>;
+
+    /// Get all province
+    async fn get_province(
+        &self,
+        name: Option<String>,
+        context: &C) -> Result<GetProvinceResponse, ApiError>;
+
+    /// Find province by ID
+    async fn get_province_by_id(
+        &self,
+        id: uuid::Uuid,
+        context: &C) -> Result<GetProvinceByIdResponse, ApiError>;
+
+    /// Update an existing province
+    async fn update_province(
+        &self,
+        id: uuid::Uuid,
+        province: Option<models::Province>,
+        context: &C) -> Result<UpdateProvinceResponse, ApiError>;
+
 }
 
 /// API where `Context` isn't passed on every API call
@@ -434,6 +534,37 @@ pub trait ApiNoContext<C: Send + Sync> {
         parish: Option<models::Parish>,
         ) -> Result<UpdateParishResponse, ApiError>;
 
+    /// Add new province
+    async fn add_province(
+        &self,
+        province: Option<models::Province>,
+        ) -> Result<AddProvinceResponse, ApiError>;
+
+    /// Delete province by ID
+    async fn delete_province(
+        &self,
+        id: uuid::Uuid,
+        ) -> Result<DeleteProvinceResponse, ApiError>;
+
+    /// Get all province
+    async fn get_province(
+        &self,
+        name: Option<String>,
+        ) -> Result<GetProvinceResponse, ApiError>;
+
+    /// Find province by ID
+    async fn get_province_by_id(
+        &self,
+        id: uuid::Uuid,
+        ) -> Result<GetProvinceByIdResponse, ApiError>;
+
+    /// Update an existing province
+    async fn update_province(
+        &self,
+        id: uuid::Uuid,
+        province: Option<models::Province>,
+        ) -> Result<UpdateProvinceResponse, ApiError>;
+
 }
 
 /// Trait to extend an API to make it easy to bind it to a context.
@@ -618,6 +749,57 @@ impl<T: Api<C> + Send + Sync, C: Clone + Send + Sync> ApiNoContext<C> for Contex
         self.api().update_parish(id, parish, &context).await
     }
 
+    /// Add new province
+    async fn add_province(
+        &self,
+        province: Option<models::Province>,
+        ) -> Result<AddProvinceResponse, ApiError>
+    {
+        let context = self.context().clone();
+        self.api().add_province(province, &context).await
+    }
+
+    /// Delete province by ID
+    async fn delete_province(
+        &self,
+        id: uuid::Uuid,
+        ) -> Result<DeleteProvinceResponse, ApiError>
+    {
+        let context = self.context().clone();
+        self.api().delete_province(id, &context).await
+    }
+
+    /// Get all province
+    async fn get_province(
+        &self,
+        name: Option<String>,
+        ) -> Result<GetProvinceResponse, ApiError>
+    {
+        let context = self.context().clone();
+        self.api().get_province(name, &context).await
+    }
+
+    /// Find province by ID
+    async fn get_province_by_id(
+        &self,
+        id: uuid::Uuid,
+        ) -> Result<GetProvinceByIdResponse, ApiError>
+    {
+        let context = self.context().clone();
+        self.api().get_province_by_id(id, &context).await
+    }
+
+    /// Update an existing province
+    async fn update_province(
+        &self,
+        id: uuid::Uuid,
+        province: Option<models::Province>,
+        ) -> Result<UpdateProvinceResponse, ApiError>
+    {
+        let context = self.context().clone();
+        self.api().update_province(id, province, &context).await
+    }
+
 }
 
 
