@@ -4,6 +4,45 @@ use crate::models;
 #[cfg(any(feature = "client", feature = "server"))]
 use crate::header;
 
+/// Enumeration of values.
+/// Since this enum's variants do not hold data, we can easily define them them as `#[repr(C)]`
+/// which helps with FFI.
+#[allow(non_camel_case_types)]
+#[repr(C)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, serde::Serialize, serde::Deserialize)]
+#[cfg_attr(feature = "conversion", derive(frunk_enum_derive::LabelledGenericEnum))]
+pub enum Code {
+    #[serde(rename = "HA_NOI")]
+    HA_NOI,
+    #[serde(rename = "HUE")]
+    HUE,
+    #[serde(rename = "SAI_GON")]
+    SAI_GON,
+}
+
+impl std::fmt::Display for Code {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        match *self {
+            Code::HA_NOI => write!(f, "{}", "HA_NOI"),
+            Code::HUE => write!(f, "{}", "HUE"),
+            Code::SAI_GON => write!(f, "{}", "SAI_GON"),
+        }
+    }
+}
+
+impl std::str::FromStr for Code {
+    type Err = String;
+
+    fn from_str(s: &str) -> std::result::Result<Self, Self::Err> {
+        match s {
+            "HA_NOI" => std::result::Result::Ok(Code::HA_NOI),
+            "HUE" => std::result::Result::Ok(Code::HUE),
+            "SAI_GON" => std::result::Result::Ok(Code::SAI_GON),
+            _ => std::result::Result::Err(format!("Value not valid: {}", s)),
+        }
+    }
+}
+
 /// An area controlled by a diocese
 #[derive(Debug, Clone, PartialEq, serde::Serialize, serde::Deserialize)]
 #[cfg_attr(feature = "conversion", derive(frunk::LabelledGeneric))]
@@ -513,45 +552,6 @@ impl std::convert::TryFrom<hyper::header::HeaderValue> for header::IntoHeaderVal
 }
 
 
-/// Enumeration of values.
-/// Since this enum's variants do not hold data, we can easily define them them as `#[repr(C)]`
-/// which helps with FFI.
-#[allow(non_camel_case_types)]
-#[repr(C)]
-#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, serde::Serialize, serde::Deserialize)]
-#[cfg_attr(feature = "conversion", derive(frunk_enum_derive::LabelledGenericEnum))]
-pub enum EcclesiasticalProvince {
-    #[serde(rename = "HA_NOI")]
-    HA_NOI,
-    #[serde(rename = "HUE")]
-    HUE,
-    #[serde(rename = "SAI_GON")]
-    SAI_GON,
-}
-
-impl std::fmt::Display for EcclesiasticalProvince {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        match *self {
-            EcclesiasticalProvince::HA_NOI => write!(f, "{}", "HA_NOI"),
-            EcclesiasticalProvince::HUE => write!(f, "{}", "HUE"),
-            EcclesiasticalProvince::SAI_GON => write!(f, "{}", "SAI_GON"),
-        }
-    }
-}
-
-impl std::str::FromStr for EcclesiasticalProvince {
-    type Err = String;
-
-    fn from_str(s: &str) -> std::result::Result<Self, Self::Err> {
-        match s {
-            "HA_NOI" => std::result::Result::Ok(EcclesiasticalProvince::HA_NOI),
-            "HUE" => std::result::Result::Ok(EcclesiasticalProvince::HUE),
-            "SAI_GON" => std::result::Result::Ok(EcclesiasticalProvince::SAI_GON),
-            _ => std::result::Result::Err(format!("Value not valid: {}", s)),
-        }
-    }
-}
-
 #[derive(Debug, Clone, PartialEq, serde::Serialize, serde::Deserialize)]
 #[cfg_attr(feature = "conversion", derive(frunk::LabelledGeneric))]
 pub struct Parish {
@@ -955,8 +955,8 @@ impl std::convert::TryFrom<hyper::header::HeaderValue> for header::IntoHeaderVal
 #[derive(Debug, Clone, PartialEq, serde::Serialize, serde::Deserialize)]
 #[cfg_attr(feature = "conversion", derive(frunk::LabelledGeneric))]
 pub struct Province {
-    #[serde(rename = "ecclesiasticalProvince")]
-    pub ecclesiastical_province: models::EcclesiasticalProvince,
+    #[serde(rename = "code")]
+    pub code: models::Code,
 
     #[serde(rename = "id")]
     #[serde(skip_serializing_if="Option::is_none")]
@@ -971,9 +971,9 @@ pub struct Province {
 }
 
 impl Province {
-    pub fn new(ecclesiastical_province: models::EcclesiasticalProvince, name: String, person_in_charge: String, ) -> Province {
+    pub fn new(code: models::Code, name: String, person_in_charge: String, ) -> Province {
         Province {
-            ecclesiastical_province: ecclesiastical_province,
+            code: code,
             id: None,
             name: name,
             person_in_charge: person_in_charge,
@@ -987,7 +987,7 @@ impl Province {
 impl std::string::ToString for Province {
     fn to_string(&self) -> String {
         let mut params: Vec<String> = vec![];
-        // Skipping ecclesiasticalProvince in query parameter serialization
+        // Skipping code in query parameter serialization
 
         // Skipping id in query parameter serialization
 
@@ -1013,7 +1013,7 @@ impl std::str::FromStr for Province {
         #[derive(Default)]
         // An intermediate representation of the struct to use for parsing.
         struct IntermediateRep {
-            pub ecclesiastical_province: Vec<models::EcclesiasticalProvince>,
+            pub code: Vec<models::Code>,
             pub id: Vec<uuid::Uuid>,
             pub name: Vec<String>,
             pub person_in_charge: Vec<String>,
@@ -1033,7 +1033,7 @@ impl std::str::FromStr for Province {
 
             if let Some(key) = key_result {
                 match key {
-                    "ecclesiasticalProvince" => intermediate_rep.ecclesiastical_province.push(<models::EcclesiasticalProvince as std::str::FromStr>::from_str(val).map_err(|x| format!("{}", x))?),
+                    "code" => intermediate_rep.code.push(<models::Code as std::str::FromStr>::from_str(val).map_err(|x| format!("{}", x))?),
                     "id" => intermediate_rep.id.push(<uuid::Uuid as std::str::FromStr>::from_str(val).map_err(|x| format!("{}", x))?),
                     "name" => intermediate_rep.name.push(<String as std::str::FromStr>::from_str(val).map_err(|x| format!("{}", x))?),
                     "personInCharge" => intermediate_rep.person_in_charge.push(<String as std::str::FromStr>::from_str(val).map_err(|x| format!("{}", x))?),
@@ -1047,7 +1047,7 @@ impl std::str::FromStr for Province {
 
         // Use the intermediate representation to return the struct
         std::result::Result::Ok(Province {
-            ecclesiastical_province: intermediate_rep.ecclesiastical_province.into_iter().next().ok_or("ecclesiasticalProvince missing in Province".to_string())?,
+            code: intermediate_rep.code.into_iter().next().ok_or("code missing in Province".to_string())?,
             id: intermediate_rep.id.into_iter().next(),
             name: intermediate_rep.name.into_iter().next().ok_or("name missing in Province".to_string())?,
             person_in_charge: intermediate_rep.person_in_charge.into_iter().next().ok_or("personInCharge missing in Province".to_string())?,
@@ -1093,3 +1093,114 @@ impl std::convert::TryFrom<hyper::header::HeaderValue> for header::IntoHeaderVal
     }
 }
 
+
+#[derive(Debug, Clone, PartialEq, serde::Serialize, serde::Deserialize)]
+#[cfg_attr(feature = "conversion", derive(frunk::LabelledGeneric))]
+pub struct ProvinceCollection {
+    #[serde(rename = "parishes")]
+    #[serde(skip_serializing_if="Option::is_none")]
+    pub parishes: Option<Vec<models::Province>>,
+
+}
+
+impl ProvinceCollection {
+    pub fn new() -> ProvinceCollection {
+        ProvinceCollection {
+            parishes: None,
+        }
+    }
+}
+
+/// Converts the ProvinceCollection value to the Query Parameters representation (style=form, explode=false)
+/// specified in https://swagger.io/docs/specification/serialization/
+/// Should be implemented in a serde serializer
+impl std::string::ToString for ProvinceCollection {
+    fn to_string(&self) -> String {
+        let mut params: Vec<String> = vec![];
+        // Skipping parishes in query parameter serialization
+
+        params.join(",").to_string()
+    }
+}
+
+/// Converts Query Parameters representation (style=form, explode=false) to a ProvinceCollection value
+/// as specified in https://swagger.io/docs/specification/serialization/
+/// Should be implemented in a serde deserializer
+impl std::str::FromStr for ProvinceCollection {
+    type Err = String;
+
+    fn from_str(s: &str) -> std::result::Result<Self, Self::Err> {
+        #[derive(Default)]
+        // An intermediate representation of the struct to use for parsing.
+        struct IntermediateRep {
+            pub parishes: Vec<Vec<models::Province>>,
+        }
+
+        let mut intermediate_rep = IntermediateRep::default();
+
+        // Parse into intermediate representation
+        let mut string_iter = s.split(',').into_iter();
+        let mut key_result = string_iter.next();
+
+        while key_result.is_some() {
+            let val = match string_iter.next() {
+                Some(x) => x,
+                None => return std::result::Result::Err("Missing value while parsing ProvinceCollection".to_string())
+            };
+
+            if let Some(key) = key_result {
+                match key {
+                    "parishes" => return std::result::Result::Err("Parsing a container in this style is not supported in ProvinceCollection".to_string()),
+                    _ => return std::result::Result::Err("Unexpected key while parsing ProvinceCollection".to_string())
+                }
+            }
+
+            // Get the next key
+            key_result = string_iter.next();
+        }
+
+        // Use the intermediate representation to return the struct
+        std::result::Result::Ok(ProvinceCollection {
+            parishes: intermediate_rep.parishes.into_iter().next(),
+        })
+    }
+}
+
+// Methods for converting between header::IntoHeaderValue<ProvinceCollection> and hyper::header::HeaderValue
+
+#[cfg(any(feature = "client", feature = "server"))]
+impl std::convert::TryFrom<header::IntoHeaderValue<ProvinceCollection>> for hyper::header::HeaderValue {
+    type Error = String;
+
+    fn try_from(hdr_value: header::IntoHeaderValue<ProvinceCollection>) -> std::result::Result<Self, Self::Error> {
+        let hdr_value = hdr_value.to_string();
+        match hyper::header::HeaderValue::from_str(&hdr_value) {
+             std::result::Result::Ok(value) => std::result::Result::Ok(value),
+             std::result::Result::Err(e) => std::result::Result::Err(
+                 format!("Invalid header value for ProvinceCollection - value: {} is invalid {}",
+                     hdr_value, e))
+        }
+    }
+}
+
+#[cfg(any(feature = "client", feature = "server"))]
+impl std::convert::TryFrom<hyper::header::HeaderValue> for header::IntoHeaderValue<ProvinceCollection> {
+    type Error = String;
+
+    fn try_from(hdr_value: hyper::header::HeaderValue) -> std::result::Result<Self, Self::Error> {
+        match hdr_value.to_str() {
+             std::result::Result::Ok(value) => {
+                    match <ProvinceCollection as std::str::FromStr>::from_str(value) {
+                        std::result::Result::Ok(value) => std::result::Result::Ok(header::IntoHeaderValue(value)),
+                        std::result::Result::Err(err) => std::result::Result::Err(
+                            format!("Unable to convert header value '{}' into ProvinceCollection - {}",
+                                value, err))
+                    }
+             },
+             std::result::Result::Err(e) => std::result::Result::Err(
+                 format!("Unable to convert header: {:?} to string: {}",
+                     hdr_value, e))
+        }
+    }
+}
+
